#!/usr/bin/perl -w

# Trkiye Biliim Dernei'nin szln kullanmak iin
# browser yerine perl'e ihtiyac duymay salayan program

use strict;
use Getopt::Long;
use DBI;

# Ayarlar
# $store_method: Kullanlacak saklama yntemi.
# mysql ya da csv'den biri olabilir. Her yntemin ihtiya
# duyduu ayarlar farkl:
#   csv => dbdir: Kelimeleri saklayacak CSV dosyasn bulunduracak dizin
#          table: Kelimeleri saklayacak dosya ismi
#
#   mysql => dbhost: MySQL sunucusu
#            dbase : Veritaban ismi
#            dbuser: Sunucuya balanrken kullanlacak isim
#            dbpass: Sunucu parolas
#            table : Kelimelerin saklanaca tablo

######

my $store_method = "mysql";
my $dbhost = "localhost";
my $dbase = "sozluk";
my $dbuser = "user";
my $dbpass = "pass";
my $dbdir = "/path/to/sozluk";
my $table = "tbd_csv";
die ("altrmadan nce ayarlamalar yapn ve bu satr silin!!!\n");

######

my $word = "";
my $regexp = "";
my $partial = "";
my $turkish = "";
my $help = "";
my $version = "";

Getopt::Long::Configure("bundling");
GetOptions(
	'regex|regexp|r' => \$regexp,
	'partial|p'      => \$partial,
	'turkish|t'      => \$turkish,
	'help|h'         => \$help,
	'version'        => \$version
);

if ($help) {
	print qq{Kullanimi: $0 [SECENEK] KELME
KELME'yi Trkiye Biliim Dernei szlnde ara ve sonucu gster.

  -r, --regex, --regexp   REGEX ifadesi
  -p, --partial           Btn bir kelimenin paras
  -t, --turkish           Aranacak kelime Trke
  -h, --help              Bu yazlar gster
  --version               Srm bilgisi

n tanml yntem kelimeyi ngilizce'den Trke'ye evirme ve btn
olarak aramadr. Ltfen yanllklar aersoy\@tfz.net adresine bildirin.
};
	exit
}

if ($version) {
	print "$0 v1.0\n";
	print "Alper Ersoy (aersoy\@tfz.net) tarafndan yazld\n";
	exit
}

if (scalar(@ARGV) == 0) {
	print "$0: aranacak kelime verilmedi\n";
	print "Daha fazla bilgi iin '$0 --help' yazn\n";
	exit 1;
}

if ($regexp and $partial) {
	print "Uyar: -r parametresi -p'den ncelikli, partial parametresi gzard edildi\n";
	$partial = "";
}

if ($store_method eq "mysql") {
	$DBI::dbh = DBI->connect("DBI:mysql:database=$dbase;host=$dbhost", $dbuser, $dbpass);
	foreach (@ARGV) {
		lookup_mysql($_);
		complete_lookup($_);
	}
} elsif ($store_method eq "csv") {
	$DBI::dbh = DBI->connect("DBI:CSV:f_dir=$dbdir");
	foreach (@ARGV) {
		lookup_csv($_);
		complete_lookup($_);
	}
} else {
	print "Bilinmeyen saklama yntemi: $store_method\n";
	print "\$store_method deikeni sadece 'mysql' ya da 'csv' ";
	print "deerlerinden birini alabilir!\n";
	exit 1;
}

$DBI::sth->finish;
$DBI::dbh->disconnect;
exit;

sub lookup_mysql {
	if ($turkish) {
		if ($regexp) {
			$DBI::sth = $DBI::dbh->prepare("SELECT tr, en FROM $table WHERE tr RLIKE ?");
		} elsif ($partial) {
			$_ = "%" . "$_" . "%";
			$DBI::sth = $DBI::dbh->prepare("SELECT tr, en FROM $table WHERE tr LIKE ?");
		} else {
			$DBI::sth = $DBI::dbh->prepare("SELECT tr, en FROM $table WHERE tr = ?");
		}
	} else {
		if ($regexp) {
			$DBI::sth = $DBI::dbh->prepare("SELECT en, tr FROM $table WHERE en RLIKE ?");
		} elsif ($partial) {
			$_ = "%" . "$_" . "%";
			$DBI::sth = $DBI::dbh->prepare("SELECT en, tr FROM $table WHERE en LIKE ?");
		} else {
			$DBI::sth = $DBI::dbh->prepare("SELECT en, tr FROM $table WHERE en = ?");
		}
	}
}

sub lookup_csv {
	if ($turkish) {
		if ($regexp) {
			print "CSV metodu regex aramalarn desteklemiyor!\n";
			exit;
		} elsif ($partial) {
			$_ = "%" . "$_" . "%";
			$DBI::sth = $DBI::dbh->prepare("SELECT tr, en FROM $table WHERE tr LIKE ?");
		} else {
			$DBI::sth = $DBI::dbh->prepare("SELECT tr, en FROM $table WHERE tr = ?");
		}
	} else {
		if ($regexp) {
			print "CSV metodu regex aramalarn desteklemiyor!\n";
			exit;
		} elsif ($partial) {
			$_ = "%" . "$_" . "%";
			$DBI::sth = $DBI::dbh->prepare("SELECT en, tr FROM $table WHERE en LIKE ?");
		} else {
			$DBI::sth = $DBI::dbh->prepare("SELECT en, tr FROM $table WHERE en = ?");
		}
	}
}

sub complete_lookup {
	my @data;

	print "$_:\n" if (scalar(@ARGV) > 1 and not ($regexp or $partial));

	$DBI::sth->execute($_);
	# Read the matching records and print them out
	if ($DBI::sth->rows == 0) {
		print "'$_' zerine arama sonu vermedi.\n";
		return;
	}
	if ($regexp) {
		while (@data = $DBI::sth->fetchrow_array()) {
			print "$data[0] => $data[1]\n";
		}
	} elsif ($partial) {
		while (@data = $DBI::sth->fetchrow_array()) {
			print "$data[0] => $data[1]\n";
		}
	} else {
		while (@data = $DBI::sth->fetchrow_array()) {
			print $data[1] . "\n";
		}
	}
	
	print "-" x 10 . "\n" if (scalar(@ARGV) > 1 and not ($regexp or $partial));
}
